package com.jiuqi.grid;

import java.text.DecimalFormat;
import com.jiuqi.util.chinese.ChineseDouble;
import com.jiuqi.util.StringUtils;
import java.io.Serializable;

public class NumberCellProperty implements NumberCellPropertyIntf, CellDataProcessorIntf, Serializable
{
    protected CellDataPropertyIntf cellDataProperty;
    
    public NumberCellProperty(final CellDataPropertyIntf dataProperty) {
        this.cellDataProperty = dataProperty;
    }
    
    public NumberCellProperty(final GridCell cell) {
        this(new CellDataProperty(cell));
    }
    
    public CellDataPropertyIntf getCellDataProperty() {
        return this.cellDataProperty;
    }
    
    public int getDecimal() {
        return this.cellDataProperty.getDataFlag();
    }
    
    public void setDecimal(int value) {
        if (value <= 15) {
            if (value < 0) {
                value = 0;
            }
            this.cellDataProperty.setDataFlag((byte)value);
            return;
        }
        throw new IllegalArgumentException("错误设置了数字的小数位数，应该在0-15之间！");
    }
    
    public int getPrecision() {
        return this.cellDataProperty.getDataProperty() & 0x1F;
    }
    
    public void setPrecision(final int value) {
        if (value >= 0 && value <= 32) {
            final int data = value & 0x1F;
            this.cellDataProperty.setDataProPerty((short)((this.cellDataProperty.getDataProperty() & 0xFFFFFFE0) | data));
            return;
        }
        throw new IllegalArgumentException("错误设置了数字的精度，应该在0-32之间！");
    }
    
    public boolean getThoundsMark() {
        return (this.cellDataProperty.getDataProperty() & 0x8000) != 0x0;
    }
    
    public void setThoundsMarks(final boolean value) {
        if (value) {
            this.cellDataProperty.setDataProPerty((short)(this.cellDataProperty.getDataProperty() | 0x8000));
        }
        else {
            this.cellDataProperty.setDataProPerty((short)(this.cellDataProperty.getDataProperty() & 0xFFFF7FFF));
        }
    }
    
    public boolean getBracketNegative() {
        return (this.cellDataProperty.getDataProperty() & 0x4000) != 0x0;
    }
    
    public void setBracketNegative(final boolean value) {
        if (value) {
            this.cellDataProperty.setDataProPerty((short)(this.cellDataProperty.getDataProperty() | 0x4000));
        }
        else {
            this.cellDataProperty.setDataProPerty((short)(this.cellDataProperty.getDataProperty() & 0xFFFFBFFF));
        }
    }
    
    public boolean getWarningNegative() {
        return (this.cellDataProperty.getDataProperty() & 0x2000) != 0x0;
    }
    
    public void setWarningNegative(final boolean value) {
        if (value) {
            this.cellDataProperty.setDataProPerty((short)(this.cellDataProperty.getDataProperty() | 0x2000));
        }
        else {
            this.cellDataProperty.setDataProPerty((short)(this.cellDataProperty.getDataProperty() & 0xFFFFDFFF));
        }
    }
    
    public boolean getChineseNumber() {
        return (this.cellDataProperty.getDataProperty() & 0x1000) != 0x0;
    }
    
    public void setChineseNumber(final boolean value) {
        if (value) {
            this.cellDataProperty.setDataProPerty((short)(this.cellDataProperty.getDataProperty() | 0x1000));
        }
        else {
            this.cellDataProperty.setDataProPerty((short)(this.cellDataProperty.getDataProperty() & 0xFFFFEFFF));
        }
    }
    
    public boolean getBigChineseChar() {
        return (this.cellDataProperty.getDataProperty() & 0x800) != 0x0;
    }
    
    public void setBigChineseChar(final boolean value) {
        if (value) {
            this.cellDataProperty.setDataProPerty((short)(this.cellDataProperty.getDataProperty() | 0x800));
        }
        else {
            this.cellDataProperty.setDataProPerty((short)(this.cellDataProperty.getDataProperty() & 0xFFFFF7FF));
        }
    }
    
    public boolean getIsPercent() {
        return this.cellDataProperty.getDataType() == 2 && (this.cellDataProperty.getDataProperty() & 0x400) != 0x0;
    }
    
    public void setIsPercent(final boolean value) {
        if (this.cellDataProperty.getDataType() == 2) {
            if (value) {
                this.cellDataProperty.setDataProPerty((short)(this.cellDataProperty.getDataProperty() | 0x400));
            }
            else {
                this.cellDataProperty.setDataProPerty((short)(this.cellDataProperty.getDataProperty() & 0xFFFFFBFF));
            }
        }
    }
    
    public String getEditText(final String storeText) {
        return storeText;
    }
    
    public String getShowText(String storeText) {
        if (storeText != null) {
            String showText = null;
            if (storeText == "") {
                showText = "";
            }
            else {
                storeText = StringUtils.eliminate(storeText, ',');
                double value;
                try {
                    value = Double.parseDouble(storeText);
                }
                catch (final NumberFormatException ex) {
                    return storeText;
                }
                final boolean isNegtive = value < 0.0;
                final boolean isPercent = this.getIsPercent();
                final boolean showThoundsMark = this.getThoundsMark();
                if (isNegtive) {
                    value = -value;
                }
                if (isPercent && !showThoundsMark) {
                    value *= 100.0;
                    this.setDecimal(this.getDecimal() - 2);
                }
                else if (isPercent && showThoundsMark) {
                    value *= 1000.0;
                    this.setDecimal(this.getDecimal() - 3);
                }
                int decimal = this.getDecimal();
                if (decimal == 15) {
                    decimal = 0;
                }
                value = trimDouble(value, this.getDecimal());
                if (!this.getChineseNumber()) {
                    showText = formatNumberString(value, this.getDecimal(), this.getThoundsMark());
                    if (isPercent && !showThoundsMark) {
                        showText = String.valueOf(showText) + "%";
                    }
                    if (isPercent && showThoundsMark) {
                        showText = String.valueOf(showText) + "‰";
                    }
                    if (isNegtive) {
                        if (this.getBracketNegative()) {
                            showText = "(" + showText + ")";
                        }
                        else {
                            showText = "-" + showText;
                        }
                    }
                }
                else {
                    if (isPercent && !showThoundsMark) {
                        final ChineseDouble cnDouble = new ChineseDouble(value / 100.0);
                        showText = cnDouble.getPercentForm(this.getDecimal(), true, this.getBigChineseChar());
                    }
                    if (isPercent && showThoundsMark) {
                        final ChineseDouble cnDouble = new ChineseDouble(value / 1000.0);
                        showText = cnDouble.getPermillageForm(this.getDecimal(), true, this.getBigChineseChar());
                    }
                    if (!isPercent) {
                        final ChineseDouble cnDouble = new ChineseDouble(value);
                        if (!showThoundsMark) {
                            showText = cnDouble.chineseValue(this.getDecimal(), false, this.getBigChineseChar());
                        }
                        else {
                            showText = cnDouble.chineseValue(this.getDecimal(), true, this.getBigChineseChar());
                        }
                    }
                    if (isNegtive) {
                        showText = "（负）" + showText;
                    }
                }
                if (isPercent) {
                    this.setDecimal(this.getDecimal() + (showThoundsMark ? 3 : 2));
                }
            }
            return showText;
        }
        throw new IllegalArgumentException("不能解析指向null的字符串为数字！");
    }
    
    public String parseEditText(final String editText) {
        final double value = Double.parseDouble(editText);
        return formatNumberString(value, this.getDecimal(), false);
    }
    
    public String parseShowText(final String showText) {
        String value = "";
        if (!this.getChineseNumber()) {
            value = showText;
        }
        else {
            try {
                value = ChineseDouble.valueOf(showText);
            }
            catch (final Exception ex) {
                throw new IllegalArgumentException("中文数值有错误，解析失败");
            }
        }
        final double dbValue = Double.parseDouble(value);
        return formatNumberString(dbValue, this.getDecimal(), this.getThoundsMark());
    }
    
    private static double trimDouble(final double source, int frcLength) {
        if (frcLength < 0) {
            frcLength = 0;
        }
        double des = source;
        final int multiplier = (int)Math.exp(Math.log(10.0) * frcLength);
        final double temp = (double)Math.round(source * multiplier);
        des = temp / multiplier;
        return des;
    }
    
    private static String formatNumberString(final double value, final int decimal, final boolean thoundsMark) {
        final DecimalFormat df = new DecimalFormat();
        df.setGroupingUsed(false);
        if (thoundsMark) {
            df.setGroupingSize(3);
            df.setGroupingUsed(true);
        }
        if (decimal >= 0) {
            df.setMaximumFractionDigits(decimal);
            df.setMinimumFractionDigits(decimal);
        }
        return df.format(value);
    }
}
